import re
import sys
import argparse
import collections
from path import path

from theX.image.color import Palette, ZXColor
from theX.image.picture import Picture
from theX.image.formats import Format

from theX.trdos import HobetaFile, trd, scl
from theX.tape import tap

files_written = collections.defaultdict(int)

def rgb_color(s):
    if not re.match(r'^[0-9a-fA-F]{6}$', s):
        raise TypeError()

    r = int(s[:2], 16)
    g = int(s[2:4], 16)
    b = int(s[4:], 16)

    return (r, g, b)

def process_file(bytes, src, native_name=None):
    src = path(src)
    for format_handler in Format.formats:
        handler = format_handler.detect(bytes, src)
        if handler:
            pic = handler.bitmap(palette)
            if native_name:
                print "Processing %s... [%s]" % (native_name, pic.format)
            else:
                print "Processing %s... [%s]" % (src, pic.format)

            if not args.animate:
                src = src.namebase
                n = files_written[src]
                if n != 0:
                    src += ".%03d" % n
                pic.save(dest, src, args.format, options)
                files_written[src] += 1
            else:
                pics.append(pic)

def process_hobeta(hobeta):
    bytes = hobeta.content
    src = path(hobeta.pc_name)
    process_file(bytes, src, hobeta.trdos_name)

def process_trd_or_scl(src, loader):
    files = loader(src)
    if files:
        print "Entering %s..." % src
        for hobeta in files:
            process_hobeta(hobeta)

def process_tape(src):
    files = tap.load(src)
    if files:
        print "Entering %s..." % src
        for header, block in files:
            if header is None:
                process_file(block, path('headerless'))
            else:
                process_file(block, path(header.pc_name))

def process_item(src):
    if src.ext.lower() == '.trd':
        process_trd_or_scl(src, trd.load)
    elif src.ext.lower() == '.scl':
        process_trd_or_scl(src, scl.load)
    elif src.ext.lower() in ('.tap', '.blk'):
        process_tape(src)
    else:
        if src.size < 65536:
            bytes = src.bytes()
            if HobetaFile.is_hobeta(bytes):
                hobeta = HobetaFile(bytes)
                process_hobeta(hobeta)
            else:
                process_file(bytes, src)

def process(src):
    if src.isfile():
        process_item(src)
    else:
        for item in src.walk():
            process(item)

parser = argparse.ArgumentParser(
    fromfile_prefix_chars='@',
    formatter_class=argparse.RawDescriptionHelpFormatter,
    version='ZX Image Convertor 0.4\nCopyright (c) 2oo9-1o by elf/2'
)

parser.add_argument('-f', '--format',
                    choices=('gif', 'png'), default='gif',
                    help='output format (default: %(default)s)')
parser.add_argument('-d', '--dest', default='.',
                    help='destination folder (default: current dir)')

palettes = Palette.repository.keys()
help = 'palette to use: %s or custom palette in Unreal Speccy format' % ', '.join(palettes)
parser.add_argument('-p', '--palette',
                    type=Palette, default='pulsar',
                    help=help + ' (default: %(default)s)')
parser.add_argument('--show-hidden-pixels', metavar='COLOR',
                    type=rgb_color, default=None,
                    nargs='?', const='ffa500',
                    help='hidden pixels color in hex rgb format, %(const)s if value omitted')

postprocessing = parser.add_argument_group('Post Processing')
postprocessing.add_argument('-b', '--border',
                            type=int, choices=range(8), default=0,
                            help='add border (default color: %(default)d)')
postprocessing.add_argument('-bw', '--border-width',
                            choices=('none', 'small', 'complete'), default='small',
                            help='set border width (default: %(default)s)')
postprocessing.add_argument('-x', '--scale',
                            type=float, default=1,
                            choices=(.5, 1, 2, 3),
                            help='scale images (default scale: %(default)d)')
postprocessing.add_argument('-i', '--interlace', metavar='TRANSPARENCY',
                            type=float, default=0,
                            nargs='?', const=.5,
                            help='make interlaced image with transparency, %(const)s if value omitted')
postprocessing.add_argument('--blur', action='store_true',
                            help='add light blur')

animation = parser.add_argument_group('Animation')
animation.add_argument('-a', '--animate', metavar='FILE',
                       const='theX.animation.gif', nargs='?',
                       help='create animated image (default: %(const)s)')
animation.add_argument('-e', '--delay',
                       type=int, default=100,
                       help='delay after each frame (default: %(default)d)')
animation.add_argument('-l', '--delay-last',
                       type=int, default=1000,
                       help='delay after the last frame (default: %(default)d)')

parser.add_argument('file', nargs='+',
                    help='file or folder to process')


args = parser.parse_args()

if args.animate and args.format != 'gif':
    print "Animation can be saved in GIF format only"
    sys.exit(1)

palette = args.palette
palette.hidden_pixel_color = args.show_hidden_pixels

color = palette[ZXColor(args.border)]
options = {
    'scale': args.scale,
    'interlace': args.interlace,
    'blur': args.blur,
    'border-color': color,
    'border-width': args.border_width,
}

dest = path(args.dest)
if not dest.exists():
    dest.makedirs()

pics = []
for src in args.file:
    process(path(src))

if args.animate:
    images = []

    for pic in pics:
        if len(pic.images) == 2 and pic.durations == (50, 50):
            # save only the first frame of flashing screens
            images.append(pic.images[0])
        else:
            images.extend(pic.images)

    if len(images) > 1:
        durations = [args.delay] * (len(images) - 1)
        durations.append(args.delay_last)
        opts = {'durations': durations}
        pic = Picture(*images, **opts)

        namebase = path(args.animate).namebase
        pic.save(dest, namebase, 'gif', options)
    else:
        print "Animation should contain more than one frame"
        sys.exit(1)